using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Xml;

using com.ibm.gre;

namespace SampleApp
{
	/// <summary>
	/// <p>The AllowedTargetsForm displays a list of
	/// File Plan Component Defenition types
	/// showing the type of Components that can be added
	/// as targets to the selected File Plan Component.</p>
	/// <p>
	/// This class is called from the Form1.mnuAddTreeItemTarget_Click
	/// event handler
	/// </p> 
	/// </summary>
	public class AllowedTargetsForm : System.Windows.Forms.Form
	{
		/// <summary>
		/// Constructor:
		/// </summary>
		/// <param name="list">An IRMList object containing the results of a
		/// call to the FilePlanComponentControllerEJB.getAllowedTargetList method.</param>
		public AllowedTargetsForm(IRMList list)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			m_list = list;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.listView1 = new System.Windows.Forms.ListView();
			this.columnHeader1 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader2 = new System.Windows.Forms.ColumnHeader();
			this.SuspendLayout();
			// 
			// listView1
			// 
			this.listView1.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																						this.columnHeader1,
																						this.columnHeader2});
			this.listView1.Dock = System.Windows.Forms.DockStyle.Fill;
			this.listView1.FullRowSelect = true;
			this.listView1.Location = new System.Drawing.Point(0, 0);
			this.listView1.Name = "listView1";
			this.listView1.Size = new System.Drawing.Size(408, 173);
			this.listView1.TabIndex = 0;
			this.listView1.View = System.Windows.Forms.View.Details;
			this.listView1.DoubleClick += new System.EventHandler(this.listView1_DoubleClick);
			// 
			// columnHeader1
			// 
			this.columnHeader1.Text = "Component Definition Name";
			this.columnHeader1.Width = 200;
			// 
			// columnHeader2
			// 
			this.columnHeader2.Text = "Component Definition Type";
			this.columnHeader2.Width = 200;
			// 
			// AllowedTargetsForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(408, 173);
			this.Controls.Add(this.listView1);
			this.Name = "AllowedTargetsForm";
			this.Text = "AllowedTargetsForm";
			this.Load += new System.EventHandler(this.AllowedTargetsForm_Load);
			this.ResumeLayout(false);

		}
		#endregion

		#region events
		/// <summary>
		/// Add contents of IRMList to a ListView
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void AllowedTargetsForm_Load(object sender, System.EventArgs e)
		{
			for(int i = 0; i < m_list.Count; i++)
			{
				string strFlPlnCmpntDefNm = m_list.Caption(i);
				IRMConstants.FilePlanComponentTypes nFlPlnCmpntDefTyp = (IRMConstants.FilePlanComponentTypes)Convert.ToInt32(m_list.getAttributeValue(i,"FlPlnCmpntDefTyp"));
				string strFlPlnCmpntDefTyp = nFlPlnCmpntDefTyp.ToString();
				
				ListViewItem lvItem = listView1.Items.Add(new ListViewItem(
					new string[]{strFlPlnCmpntDefNm,strFlPlnCmpntDefTyp}));
				lvItem.Tag = m_list.PrimaryKey(i);
			}
		}

		/// <summary>
		/// Saves the selected Component Definition Type
		/// and closes the dialog.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void listView1_DoubleClick(object sender, System.EventArgs e)
		{
			IEnumerator items = listView1.SelectedItems.GetEnumerator();
			items.MoveNext();
			ListViewItem lvi = (ListViewItem)items.Current;
			//Get saved search xml and leave
			m_nFlPlnCmpntDefTgtID = Convert.ToInt32(lvi.Tag);
			DialogResult = DialogResult.OK;
			this.Close();
		}
		#endregion

		/// <summary>
		/// Gets the selected Component Definition ID.
		/// </summary>
		public long FlPlnCmpntDefTgtID
		{
			get
			{
				return m_nFlPlnCmpntDefTgtID;
			}
		}

		#region MemberVariables
		private long m_nFlPlnCmpntDefTgtID;
		private IRMList m_list;

		private System.Windows.Forms.ListView listView1;
		private System.Windows.Forms.ColumnHeader columnHeader1;
		private System.Windows.Forms.ColumnHeader columnHeader2;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		#endregion
	}
}
