using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Xml;

using com.ibm.gre;

namespace SampleApp
{
	/// <summary>
	/// The QueryForm displays Records Manager Saved Queries
	/// in an easy to use format.
	/// This form demonstrates how to use the IRMQuery class.
	/// </summary>
	public class QueryForm : System.Windows.Forms.Form
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public QueryForm(Control parent, IRMQuery irmQuery, string caption, int nQueryResultOptions)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			label1.Text = caption;
			m_irmQuery = irmQuery;
			m_nQueryResultOptions = nQueryResultOptions;

			util = new Util(this,parent);

		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.btnOk = new System.Windows.Forms.Button();
			this.label1 = new System.Windows.Forms.Label();
			this.btnCancel = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// btnOk
			// 
			this.btnOk.Location = new System.Drawing.Point(512, 16);
			this.btnOk.Name = "btnOk";
			this.btnOk.TabIndex = 4;
			this.btnOk.Text = "Okay";
			this.btnOk.Click += new System.EventHandler(this.btnOk_Click);
			// 
			// label1
			// 
			this.label1.BackColor = System.Drawing.Color.Gold;
			this.label1.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.label1.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label1.Location = new System.Drawing.Point(16, 16);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(480, 56);
			this.label1.TabIndex = 5;
			// 
			// btnCancel
			// 
			this.btnCancel.Location = new System.Drawing.Point(512, 48);
			this.btnCancel.Name = "btnCancel";
			this.btnCancel.TabIndex = 7;
			this.btnCancel.Text = "Cancel";
			this.btnCancel.Click += new System.EventHandler(this.btnCancel_Click);
			// 
			// QueryForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(600, 189);
			this.Controls.Add(this.btnCancel);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.btnOk);
			this.Name = "QueryForm";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "QueryForm";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.QueryForm_Closing);
			this.Load += new System.EventHandler(this.QueryForm_Load);
			this.ResumeLayout(false);

		}
		#endregion

		#region events
		private void QueryForm_Load(object sender, System.EventArgs e)
		{
			const int space = 1;
			const int tabWidth = 10;
			const int controlWidth = 200;
			const int topSpace = 5;
			const int fudgeFactor = 64;

			int height = btnOk.Size.Height;
			int x = label1.Location.X;
			int y = label1.Location.Y + label1.Height + topSpace;

			int count = m_irmQuery.LineCount;
			util.log("IRMQuery.LineCount = " + count);

			this.Height = label1.Height + fudgeFactor + count * (height + space);

			util.log("Display " + count + " lines in the Query Dialog");
			for (int i = 0; i < count; i++)
			{
				Line line = m_irmQuery.getLine(i);
				util.log("Getting Line object using IRMQuery.getLine(" + i + ")");
				int tab = line.Tab * tabWidth;

				Label lbl = new Label();
				lbl.Location = new Point(x + tab,y);
				lbl.AutoSize = true;
				lbl.Text = line.Caption;
				this.Controls.Add (lbl);

				switch(line.ControlType)
				{
					//TODO many more cases could be handled
					case (int)IRMConstants.ProfileControlType.SINGLELINE:
						TextBox txtBox = new TextBox();
						txtBox.Location = new Point(x + tab + lbl.Width,y);
						txtBox.Width = controlWidth;
						txtBox.Tag = line.Key;
						util.log("Add a TextBox with Key value: " + line.Key);
						if(line.Key == "SCOPE")
						{
							txtBox.Text = m_irmQuery.getFieldValue("SCOPE");
							util.log("Add a value for this using IRMQuery.getFieldValue(\"SCOPE\") " + txtBox.Text);
						}
						this.Controls.Add (txtBox);
						break;
					case (int)IRMConstants.ProfileControlType.DROPLIST:
						ComboBox droplist = new ComboBox();
						droplist.DropDownStyle = ComboBoxStyle.DropDownList;
						droplist.Location = new Point(x + tab + lbl.Width,y);
						droplist.Width = controlWidth;
						droplist.Tag = line.Key;
						util.log("Add a DropList with Key value: " + line.Key);

						//TODO expand this for all types
						if(line.PickListId != 0)
						{
							util.log("Populate the DropList with values from PickList: " + line.PickListId);

							string xml = m_irmQuery.ClientSession.getComponentDesignController().getPickListEntryList(line.PickListId,0,0);
							
							util.log("Call the ComponentDesignController method getPickListEntryList for XML values\r\n" + xml);

							XmlDocument doc = new XmlDocument();
							doc.LoadXml(xml);
							XmlElement root = doc.DocumentElement;
							XmlNodeList items = root.ChildNodes;
							for(int j = 0; j < items.Count; j++)
							{
								string strPckLstEntVal = items.Item(j).Attributes.GetNamedItem("PckLstEntVal").Value;
								droplist.Items.Add(strPckLstEntVal);
							}
						}
						this.Controls.Add(droplist);
						break;
					case (int)IRMConstants.ProfileControlType.COMBOBOX:
						ComboBox combo = new ComboBox();
						combo.Location = new Point(x + tab + lbl.Width,y);
						combo.Width = controlWidth;
						combo.Tag = line.Key;

						util.log("Add a ComboBox with Key value: " + line.Key);

						this.Controls.Add(combo);
						break;
					default:
						//TODO determine appropriate action here.
						break;

				}

				y += height + space;
			}
		}

		private void QueryForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			if(!valid)
			{
				e.Cancel = true;
				valid = true;
			}
		}

		private void btnCancel_Click(object sender, System.EventArgs e)
		{
			DialogResult = DialogResult.Cancel;
		}

		private void btnOk_Click(object sender, System.EventArgs e)
		{
			try
			{
				for(int i = 0; i < this.Controls.Count; i++)
				{
					if(i == 0)
					{
						util.log("Set the modified Query values");
					}
					string key = (string)this.Controls[i].Tag;
					if(key != null)
					{
						string val = this.Controls[i].Text;
						//m_irmQuery.setFieldValue returns false if the value is not of the right type
						//or cannot be found
						//TODO logic should be improved
 
						util.log("Call IRMQuery.setFieldValue(" + key + ", " + val + ")");
						if(!m_irmQuery.setFieldValue(key,val))
						{
							MessageBox.Show("Invaled value " + this.Controls[i].Text);
							valid = false;
							DialogResult = DialogResult.Abort;
							return;
						}
					}
				}
				//TODO this should be set at the Form1 level 
				util.log("Set IRMQuery.QueryResultOptions = " + m_nQueryResultOptions);
				m_irmQuery.QueryResultOptions = m_nQueryResultOptions;

				int nBlockStart = 0;
				int nBlockSize = 0;
				m_irmQuery.runQuery(nBlockStart,nBlockSize);

				//TODO you can display an input form to get block values				
/*				
				InputForm frm = new InputForm("BlockStart","BlockSize");
				if(frm.ShowDialog(this) == DialogResult.OK)
				{
					if(frm.TextBox1.Length != 0)
					{
						nBlockStart = Convert.ToInt32(frm.TextBox1);
					}
					if(frm.TextBox2.Length != 0)
					{
						nBlockSize = Convert.ToInt32(frm.TextBox2);
					}
					util.log("Call IRMQuery.runQuery(" + nBlockStart + ", " + nBlockSize + ")");
					m_irmQuery.runQuery(nBlockStart,nBlockSize);
				}
*/

			}
			catch(System.Net.WebException we)
			{
				MessageBox.Show(we.Message);
				DialogResult = DialogResult.Abort;
				valid = false;
				return;
			}
			catch(System.Web.Services.Protocols.SoapException s)
			{
				MessageBox.Show(s.Message);
				DialogResult = DialogResult.Abort;
				valid = false;
				return;
			}
			valid = true;
			DialogResult = DialogResult.OK;
		}
		#endregion

		public IRMQuery getIRMQuery()
		{
			return m_irmQuery;
		}

		public IRMBatchQuery getIRMBatchQuery()
		{
			return (IRMBatchQuery)m_irmQuery;
		}

		#region MemberVariables
		private Util util;

		private bool valid = true;
		private IRMQuery m_irmQuery;
		private int m_nQueryResultOptions;
		
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Button btnOk;
		private System.Windows.Forms.Button btnCancel;
		#endregion
	}
}
