/*
 * Created on Jan 30, 2005
 */
package com.ibm.gre.samples.collectionhelpers;

import org.w3c.dom.Node;
import com.ibm.gre.engine.util.XMLUtil;

/**
 * The Column Class encapsulate the standard attribute values
 * for each Item element contained in the MetaData element
 * from a Records Manager Query operation.
 * These values describe each column that is returned for the query.
 * See the documentation of the runQuery method in the QueryControllerEJB
 * class for more information on the XML structure. 
 */
public class Column {

	/**
	 * Constructor
	 * @param node This is an XML Item element from the MetaData section
	 * of a query result string.
	 * @throws Exception
	 */
	public Column(Node node) throws Exception
	{
		m_strFlPlnCmpntAttrDefCapt = XMLUtil.getValue(node.getAttributes().getNamedItem("FlPlnCmpntAttrDefCapt"));
		m_strFlPlnCmpntAttrDefColNm = XMLUtil.getValue(node.getAttributes().getNamedItem("FlPlnCmpntAttrDefColNm"));
		String temp = XMLUtil.getValue(node.getAttributes().getNamedItem("FlPlnCmpntAttrDefDatTyp"));
		m_nFlPlnCmpntAttrDefDatTyp = Byte.parseByte(temp);
		Node testNode = node.getAttributes().getNamedItem("FlPlnCmpntAttrDefNo");
		if(testNode == null)
		{
			m_nFlPlnCmpntAttrDefNo = -999; //Indicates the column does not have a File Plan Component Attribute Definition Number
		}
		else
		{
			temp = XMLUtil.getValue(testNode);
			m_nFlPlnCmpntAttrDefNo = temp == "" ? -999 : Integer.parseInt(temp);   
		}
		m_nFlPlnCmpntDefID = Integer.parseInt(XMLUtil.getValue(node.getAttributes().getNamedItem("FlPlnCmpntDefID")));
		testNode = node.getAttributes().getNamedItem("PckLstID");
		if(testNode == null)
		{
			m_nPckLstID = 0; //Indicates the column does not have a Pick List
		}
		else
		{
			temp = XMLUtil.getValue(testNode);
			m_nPckLstID = temp == "" ? 0 : Integer.parseInt(temp);   
		}
	}

	/**
	 * Get the suggested percentage of the total width for all Columns
	 * that this column should take up.
	 * @return int
	 */
	public int getPercentWidth()
	{
		return m_nPercentWidth;
	}
	
	/**
	 * Set the suggested percentage of the total width for all Columns
	 * that this column should take up.
	 * @param val The percentage value.
	 */
	public void	setPercentWidth(int val)
	{
		m_nPercentWidth = val;
	}

	/**
	 * The Caption that should be displayed for this column.
	 * @return String Caption
	 */
	public String getFlPlnCmpntAttrDefCapt()
	{
		return m_strFlPlnCmpntAttrDefCapt;
	}

	/**
	 * The unique name for the Component Attribute represented by the column.
	 * @return String File Plan Attribute Column Name
	 */
	public String getFlPlnCmpntAttrDefColNm()
	{
		return m_strFlPlnCmpntAttrDefColNm;
	}

	/**
	 * The Records Manager data type for the column.
	 * @return byte
	 */
	public byte getFlPlnCmpntAttrDefDatTyp()
	{
		return m_nFlPlnCmpntAttrDefDatTyp;
	}

	/**
	 * File Plan Component Attribute Definition Number
	 * @return int
	 */
	public int getFlPlnCmpntAttrDefNo()
	{
		return m_nFlPlnCmpntAttrDefNo;
	}
		
	/**
	 * The Component Definition Id of the Component
	 * being reported on.
	 * @return int
	 */
	public int getFlPlnCmpntDefID()
	{
		return m_nFlPlnCmpntDefID;
	}
		
	/**
	 * The Id of a PickList that values for
	 * this column can be selected from.
	 * @return int PickListId
	 */
	public int getPckLstID()
	{
		return m_nPckLstID;
	}

	private String m_strFlPlnCmpntAttrDefCapt;
	private String m_strFlPlnCmpntAttrDefColNm;
	private byte m_nFlPlnCmpntAttrDefDatTyp;
	private int m_nFlPlnCmpntAttrDefNo;
	private int m_nFlPlnCmpntDefID;
	private int m_nPckLstID;
	private int m_nPercentWidth = 10; //NOTE the UI developer must set this value the default will not be realistic.

}
