/*
 * Created on Dec 30, 2004
 */
package com.ibm.gre.samples.collectionhelpers;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashMap;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.ibm.gre.engine.util.AttributeControlMask;
import com.ibm.gre.engine.util.AttributeDataTypes;
import com.ibm.gre.engine.util.ProfileControlType;
import com.ibm.gre.engine.util.QueryResultOptions;
import com.ibm.gre.engine.util.RecordManagerException;
import com.ibm.gre.engine.util.XMLUtil;

/**
 * The IRMQuery class forms the basis for Easy Inteligent Query dialogs.
 * IRMQuery objects are constructed using QueryControllerEJB saved query XML values.
 * The class contains methods to assist in displaying Query Forms.
 * There are methods to execute Queries with modified values.
 */
public class IRMQuery {

	private int m_fieldCount = 1; //appended to field name then incremented to ensure unique field name keys		
	private int m_FlPlnCmpntDefID;
	private String m_QueryNm;
	private String m_strScope;

	private Document m_doc;
	private Element m_root;
	private Element m_QueryImage;
	private Document m_ReportTemplate;
	private Element m_Query;
	private Element m_Condition;
	private Element m_BlockSize;
	private Element m_BlockStart;

	protected Element m_rootReportTemplate;

	protected byte m_tab = 0;
	protected int m_nQueryType;

	protected HashMap m_fieldvalues;
	protected ArrayList m_DisplayItems;

	protected boolean m_addDisplayInfo;
	protected ClientSession m_session;
	protected String m_strResultList;
	protected int m_totalResults;
	protected int m_nQueryResultOptions = QueryResultOptions.INCLUDE_COLUMNS;
	
	protected boolean m_bIsInitialized = false;
	
	protected void print(String str){
		System.out.println(str);
	}


	/**
	 * Constructor
	 * @param queryXML he XML string containing the values making up the Query.
	 * This string can be a saved query value returned by the QueryControllerEJB.getQueryById method.
	 * @param session A ClientSession object containing references to the Records Manager API objects.
	 * @param strScope The full path string indicating the scope of the query.
	 * @param addDisplayInfo Set to true if you wish to use this object to generate display information.
	 * @throws RecordManagerException
	 * @throws Exception
	 */
	public IRMQuery(String queryXML, ClientSession session,String strScope , boolean addDisplayInfo)
	throws RecordManagerException, Exception
	{
		m_session = session;
		m_strScope = strScope;
		m_addDisplayInfo = addDisplayInfo;

		m_totalResults = 0;

		m_doc = XMLUtil.getDocument(queryXML);
		m_root = m_doc.getDocumentElement();
			
		String temp = XMLUtil.getValue(GetFieldValue("FlPlnCmpntDefID"));

		m_FlPlnCmpntDefID = Integer.parseInt(temp);
		
		m_QueryNm = XMLUtil.getValue(GetFieldValue("QueryNm"));
			
		m_QueryImage = GetFieldValue("QueryImage");

		String xml = XMLUtil.getValue(m_QueryImage);
		//print("xml = " + xml);
		m_ReportTemplate = XMLUtil.getDocument(xml);
		
		m_rootReportTemplate = m_ReportTemplate.getDocumentElement();

		m_BlockSize = (Element)XMLUtil.getNodeByXPath(m_rootReportTemplate,"BlockSize");
		
		m_BlockStart = (Element)XMLUtil.getNodeByXPath(m_rootReportTemplate,"BlockStart");
			
		m_Query = (Element)XMLUtil.getNodeByXPath(m_rootReportTemplate,"Query");
			
		m_Condition = (Element)XMLUtil.getNodeByXPath(m_Query,"Condition");

		m_fieldvalues = new HashMap();

		m_DisplayItems = null;
		if(m_addDisplayInfo)
		{
			m_DisplayItems = new ArrayList();
		}
		//Get the Query Type
		temp = XMLUtil.getValue(GetFieldValue("QueryType"));
		m_nQueryType = Integer.parseInt(temp);
	}

	
	/**
	 * Call the Initialize method immediately after the class has been constructed
	 * to generate the internal representation of the query.
	 * @throws RemoteException
	 * @throws RecordManagerException
	 * @throws Exception
	 */
	public void Initialize() throws RemoteException, RecordManagerException, Exception
	{
		Element scope = (Element)XMLUtil.getNodeByXPath(m_rootReportTemplate,"Scope");
		XMLUtil.setValue(scope,m_strScope);
		String caption = "Looking under the following locations"; 
		Line line = new Line(scope,AttributeDataTypes.STRING,"SCOPE",caption,0,LineType.SCOPE,ProfileControlType.SINGLELINE,m_tab);
		m_fieldvalues.put("SCOPE",line);

		if(m_addDisplayInfo)
		{
			m_DisplayItems.add(line);
		}
		makeProfileFields(m_Condition, m_FlPlnCmpntDefID);
		m_bIsInitialized = true;
	}

	
	/**
	 * Calls the QueryController runQuery using the mofified query values.
	 * Saves the result internally.
	 * Call getResultList to retreive the XML return value.
	 * @param nBlockStart The index value return a block of values from. 0 indicates the first value.
	 * @param nBlockSize The number of values to return. 0 indicates return all values.
	 * @return Always returns 100 meaning 100% complete.
	 * @throws RemoteException
	 * @throws RecordManagerException
	 * @throws Exception
	 */
	public int runQuery(int nBlockStart, int nBlockSize)throws RemoteException, RecordManagerException, Exception
	{
		setBlockStart(nBlockStart);
		setBlockSize(nBlockSize);
		String xml = m_session.getQueryController().runQuery(getModifiedQuery(),QueryResultOptions.COUNT_ONLY);
			
		Document doc = XMLUtil.getDocument(xml);
		Element root = doc.getDocumentElement();
		Node node = XMLUtil.getNodeByXPath(root,"Item");
		String temp = XMLUtil.getValue(node.getAttributes().getNamedItem("RecordCount"));
		m_totalResults = Integer.parseInt(temp);
		m_strResultList = m_session.getQueryController().runQuery(getModifiedQuery(),m_nQueryResultOptions);
		return 100;
	}

	/**
	 * Gets the queryResultOption paramater for the QueryController runQuery method.
	 * The default value is com.ibm.gre.engine.util.QueryResultOptions.INCLUDE_COLUMNS.
	 * @return int
	 */
	public int getQueryResultOptions() throws Exception
	{
		checkStatus();
		return m_nQueryResultOptions;
	}
	
	/**
	 * Return the total result count for the query.
	 * @return int
	 */
	public int getTotalResults() throws Exception
	{
		checkStatus();
		return m_totalResults;
	}

	/**
	 * Return the XML string result from the last call to
	 * the QueryController runQuery method.
	 * @return XML formatted String.
	 */
	public String getResultList() throws Exception
	{
		checkStatus();
		return m_strResultList;
	}

	/**
	 * Get the Block Start value.
	 * @return int
	 */
	public int getBlockStart()
	{
		return Integer.parseInt(m_BlockStart.getNodeValue());
	}
	
	/**
	 * Set the Block Start where 0 is the first element.
	 * @param value
	 */
	public void	setBlockStart(int value)
	{
		XMLUtil.setValue(m_BlockStart,Integer.toString(value));
	}

	/**
	 * Gets the value indicating the number of elements that will be returned.
	 * @return int
	 */
	public int getBlockSize()
	{
		return Integer.parseInt(m_BlockSize.getNodeValue());
	}

	/**
	 * Sets the value indicating the number of elements that will be returned.
	 * @param value
	 */
	public void	setBlockSize(int value)
	{
		XMLUtil.setValue(m_BlockSize,Integer.toString(value));
	}

	/**
	 * The encapsulated ClientSession.
	 * @return ClientSession
	 */
	public ClientSession getSession()
	{
		return m_session;
	}

	/**
	 * The Query Type.
	 * @return See com.ibm.gre.engine.types.QueryTypes
	 */
	public int getQueryType()
	{
		return m_nQueryType;
	}

	
	/**
	 * The total number of lines that will be displayed
	 * to draw a Query Form.
	 * @return int
	 */
	public int getLineCount() throws Exception
	{
		checkStatus();
		checkDisplayStatus();
		return m_DisplayItems.size();
	}
	
	/**
	 * Gets the Line object for the specified index.
	 * @param index
	 * @return Line
	 */
	public Line getLine(int index) throws Exception
	{
		checkStatus();
		checkDisplayStatus();
		return (Line)m_DisplayItems.get(index);
	}

	/**
	 * Sets the XML element value for the element named by the unique key value.
	 * @param key The unique key identifying the element to set the value for.
	 * @param val The value to set.
	 * @return true if the value was set.
	 */
	public boolean setFieldValue(String key, String val) throws Exception
	{
		checkStatus();
		if(m_fieldvalues.containsKey(key))
		{
			Line line = (Line)m_fieldvalues.get(key);
			if(validateByType(line.getFlPlnCmpntAttrDefDatTyp(),val))
			{
				line.setFieldElement(val);
				return true;
			}
			return false;
		}
		return false;
	}

	/**
	 * Gets the string value for the XML element represented by the unique key
	 * @param key The unique value identifying the XML element to get the value for.
	 * @return The value of the XML element or "" if not found.
	 * @throws Exception
	 */
	public String getFieldValue(String key) throws Exception
	{
		checkStatus();
		if(m_fieldvalues.containsKey(key))
		{
			Line line = (Line)m_fieldvalues.get(key);
			return XMLUtil.getValue(((Element)line.getFieldElement()));
		}
		return "";
	}

	
	/**
	 * The XML string representing a new Saved Query with modified values.
	 * @return String
	 */
	public String getModifiedQuery() throws Exception
	{
		checkStatus();
		String data = XMLUtil.getXML(m_rootReportTemplate);
		m_QueryImage.replaceChild(m_doc.createCDATASection(data),m_QueryImage.getFirstChild());
		return XMLUtil.getXML(m_root);
	}

	/**
	 * The Condition element of the saved query.
	 * @return XML formatted String.
	 */
	public String getCondition()
	{
		return XMLUtil.getXML(m_Condition);
	}

	/**
	 * The Query element of the saved query
	 * @return XML formatted String.
	 */
	public String getQuery()
	{
		return XMLUtil.getXML(m_Query);
	}

	/**
	 * The ReportTemplate of the saved query.
	 * @return XML formatted String.
	 */
	public String getReportTemplate()
	{
		return XMLUtil.getXML(m_ReportTemplate);
	}

	/**
	 * The File Plan Component Definition Name for the saved query.
	 * @return String name
	 * @throws RemoteException
	 * @throws RecordManagerException
	 * @throws Exception
	 */
	public String getFlPlnCmpntDefName() throws RemoteException, RecordManagerException, Exception
	{
		return getComponentName(m_FlPlnCmpntDefID);
	}
	
	/**
	 * This method should be completed to validate all different
	 * Records Manager Attribute data Types.
	 * @param nFlPlnCmpntAttrDefDatTyp
	 * @param val
	 * @return boolean
	 */
	private boolean validateByType(int nFlPlnCmpntAttrDefDatTyp, String val)
	{
		try 
		{
			switch (nFlPlnCmpntAttrDefDatTyp)
			{
				//TODO add validation for all types
				case AttributeDataTypes.INTEGER:
					Integer.parseInt(val);
					return true;
				default:
					return true;
			}
		}
		catch (Exception e) 
		{
			e.printStackTrace();
			return false;
		}
	}

	/**
	 * Recusively process all the elements contained in the Condition element
	 * and any nested Condition elements. 
	 * @param condition The current Condition Element to proccess.
	 * @param nFlPlnCmpntDefID The Component Definition that the current condition represents.
	 * @throws RemoteException
	 * @throws RecordManagerException
	 * @throws Exception
	 */
	private void makeProfileFields(Element condition, int nFlPlnCmpntDefID)
	throws RemoteException, RecordManagerException, Exception
	{
		if(m_addDisplayInfo)
		{
			String caption = "Find all the " + getComponentName(m_FlPlnCmpntDefID) + "s that meet the following criteria:"; 
			Line line = new Line(null,0,"",caption,0,LineType.NONE,ProfileControlType.GROUPHEADER,m_tab);
			m_DisplayItems.add(line);
		}

		//Get the Columns list values so we can expand the information for Field Expressions
		String xml = m_session.getQueryController().getReportColumnsList(nFlPlnCmpntDefID);
		Document doc = XMLUtil.getDocument(xml);
		Element root = doc.getDocumentElement();
			
		NodeList list = condition.getChildNodes();
		for(int i = 0; i < list.getLength(); i++)
		{
			String elementName = list.item(i).getNodeName();
			if(elementName == "FieldExpression")
			{
				Element fieldExpression = (Element)list.item(i);
				Element fieldName = (Element)XMLUtil.getNodeByXPath(fieldExpression, "FieldName");
				Element fieldValue = (Element)XMLUtil.getNodeByXPath(fieldExpression,"FieldValue");
				Element fieldOperator = (Element)XMLUtil.getNodeByXPath(fieldExpression,"FieldOperator");
				String strFlPlnCmpntAttrDefColNm = XMLUtil.getValue(fieldName);
				String key = strFlPlnCmpntAttrDefColNm + "_" + m_fieldCount++;
				
				String caption = getCaption(root,strFlPlnCmpntAttrDefColNm);
					
				String operation = fieldOperator.getNodeValue();
				int lineType = makeLineType(operation);
				operation = getOperation(lineType);
					
				caption = caption + " " + operation;

				int nFlPlnCmpntAttrDefDatTyp = getDataType(root,strFlPlnCmpntAttrDefColNm);
				int nPickListId = getPickListId(root,strFlPlnCmpntAttrDefColNm);
				int nAttrCntrlMsk = getAttrCntrlMsk(root,strFlPlnCmpntAttrDefColNm);

				int controlType = ProfileControlType.SINGLELINE;
				if(nPickListId != 0)
				{
					if(AttributeControlMask.isLimitToList(nAttrCntrlMsk))
					{
						controlType = ProfileControlType.DROPLIST;
					}
					else
					{
						controlType = ProfileControlType.COMBOBOX;
					}
				}
				if(lineType == LineType.IS_NULL)
				{
					controlType = ProfileControlType.BLANKLINE;
				}
					
				Line line = new Line(fieldValue,nFlPlnCmpntAttrDefDatTyp,key,caption,nPickListId,lineType,controlType,m_tab);
					
				if(lineType != LineType.IS_NULL)
				{
					//save the fieldValue element to receive new value
					m_fieldvalues.put(key,line);
				}

				if(m_addDisplayInfo)
				{
					m_DisplayItems.add(line);
				}

			}
			else if(elementName == "InterFieldOperator")
			{
				String interFieldOperator = XMLUtil.getValue(list.item(i));
				int lineType = makeLineType(interFieldOperator);
					
				//Right bracket should be placed directly below preceding tab
				if(lineType == LineType.RB)
				{
					m_tab--;
				}
					
				Line line = new Line(null,0,"0",interFieldOperator,0,lineType,ProfileControlType.BLANKLINE,m_tab);
					
				//All lines after Left Bracket should be tabbed
				if(lineType == LineType.LB)
				{
					m_tab++;
				}

				if(m_addDisplayInfo)
				{
					m_DisplayItems.add(line);
				}
			}
			else if(elementName == "FunctionExpression")
			{
				//FunctionExpression
				if(list.item(i).hasChildNodes())
				{
					Node node = XMLUtil.getNodeByXPath(list.item(i),"ReportOn");
					String temp = XMLUtil.getValue(node);
					int localFlPlnCmpntDefID = Integer.parseInt(temp);
					if(m_addDisplayInfo)
					{
						String strOperator = XMLUtil.getValue(XMLUtil.getNodeByXPath(list.item(i),"FunctionName"));
						int lineType = makeLineType(strOperator);
						String caption = "Filter results on " + getComponentName(localFlPlnCmpntDefID) + " " + getOperation(lineType);
						Line line = new Line(null,0,"0",caption,0,lineType,ProfileControlType.GROUPHEADER,m_tab);
						m_DisplayItems.add(line);
						//Add Left Bracket
						line = new Line(null,0,"0","{",0,0,ProfileControlType.BLANKLINE,m_tab++);
						m_DisplayItems.add(line);
					}
					//Recusively process all the elements contained in the Condition element
					//and any nested Condition elements. 
					Element localCondition = (Element)XMLUtil.getNodeByXPath(list.item(i),"Condition");
					makeProfileFields(localCondition,localFlPlnCmpntDefID);

					//Add footer item here
					if(m_addDisplayInfo)
					{
						//Add Right Bracket
						Line line = new Line(null,0,"0","}",0,0,ProfileControlType.BLANKLINE,--m_tab);
						m_DisplayItems.add(line);
					}
				}
			}
		}
	}

	private int makeLineType(String strOperator)
	{
		if(strOperator == ">")
		{
			return (int)LineType.GT;
		}
		else if(strOperator == "<")
		{
			return (int)LineType.LT;
		}
		else if(strOperator == "=")
		{
			return (int)LineType.EQ;
		}
		else if(strOperator == ">=")
		{
			return (int)LineType.GTEQ;
		}
		else if(strOperator == "<=")
		{
			return (int)LineType.LTEQ;
		}
		else if(strOperator == "LIKE")
		{
			return (int)LineType.LIKE;
		}
		else if(strOperator == "NOT LIKE")
		{
			return (int)LineType.NOT_LIKE;
		}
		else if(strOperator == "IS NULL")
		{
			return (int)LineType.IS_NULL;
		}
		else if(strOperator == "(")
		{
			return (int)LineType.LB;
		}
		else if(strOperator == "AND")
		{
			return (int)LineType.AND;
		}
		else if(strOperator == "OR")
		{
			return (int)LineType.OR;
		}
		else if(strOperator == "NOT")
		{
			return (int)LineType.NOT;
		}
		else if(strOperator == ")")
		{
			return (int)LineType.RB;
		}
		else if(strOperator == "hasChild")
		{
			return (int)LineType.HAS_CHILD;
		}
		else if(strOperator == "isChildOf")
		{
			return (int)LineType.IS_CHILD_OF;
		}
		else if(strOperator == "hasPart")
		{
			return (int)LineType.HAS_PART;
		}
		else if(strOperator == "isPartOf")
		{
			return (int)LineType.IS_PART_OF;
		}
		else
		{
			return (int)LineType.NONE;
		}
	}

	private String getOperation(int lt)
	{
		switch(lt)
		{
			case LineType.GT:
				return "is greater than";
			case LineType.LT:
				return "is less than";
			case LineType.EQ:
				return "is equal to";
			case LineType.GTEQ:
				return "is greater than or equal to";
			case LineType.LTEQ:
				return "is less than or equal to";
			case LineType.LIKE:
				return "is Like";
			case LineType.NOT_LIKE:
				return "is Not Like";
			case LineType.IS_NULL:
				return "has no value";
			case LineType.HAS_CHILD:
				return "children matching the following conditions";
			case LineType.IS_CHILD_OF:
				return "parents matching the following conditions";
			case LineType.HAS_PART:
				return "that have a partition matching the following conditions";
			case LineType.IS_PART_OF:
				return "that belong to a partition that matches the following conditions";
			default:
				return "";
		}
	}

	private Element GetFieldValue(String strFlPlnCmpntAttrDefColNm)
	{
		String xpath = "/Fields/Field[@FlPlnCmpntAttrDefColNm='" + strFlPlnCmpntAttrDefColNm + "']";
		Element lm = (Element)XMLUtil.getNodeByXPath(m_root, xpath);
		return lm;
	}
	//This method gets the Caption given a column name. It is by no means optimized
	//Caption will be problematic in a multi language environment
	//There will need to be a way to map default captions to their translations
	private String getCaption(Element root, String strFlPlnCmpntAttrDefColNm) throws Exception
	{
		String xpath = "/List/Item[@FlPlnCmpntAttrDefColNm='" + strFlPlnCmpntAttrDefColNm + "']";
		Node item = XMLUtil.getNodeByXPath(root, xpath);
		String strFlPlnCmpntAttrDefCapt = XMLUtil.getValue(item.getAttributes().getNamedItem("FlPlnCmpntAttrDefCapt"));
		return strFlPlnCmpntAttrDefCapt;
	}

	private int getDataType(Element root, String strFlPlnCmpntAttrDefColNm) throws Exception
	{
		String xpath = "/List/Item[@FlPlnCmpntAttrDefColNm='" + strFlPlnCmpntAttrDefColNm + "']";
		Node item = XMLUtil.getNodeByXPath(root, xpath);
		String strFlPlnCmpntAttrDefDatTyp = XMLUtil.getValue(item.getAttributes().getNamedItem("FlPlnCmpntAttrDefDatTyp"));
		return Integer.parseInt(strFlPlnCmpntAttrDefDatTyp);
	}

	private int getAttrCntrlMsk(Element root, String strFlPlnCmpntAttrDefColNm) throws Exception
	{
		String xpath = "/List/Item[@FlPlnCmpntAttrDefColNm='" + strFlPlnCmpntAttrDefColNm + "']";
		Node item = XMLUtil.getNodeByXPath(root, xpath);
		String strFlPlnCmpntAttrDefDatTyp = XMLUtil.getValue(item.getAttributes().getNamedItem("AttrCntrlMsk"));
		return Integer.parseInt(strFlPlnCmpntAttrDefDatTyp);
	}

	private int getPickListId(Element root, String strFlPlnCmpntAttrDefColNm) throws Exception
	{
		String xpath = "/List/Item[@FlPlnCmpntAttrDefColNm='" + strFlPlnCmpntAttrDefColNm + "']";
		Node item = XMLUtil.getNodeByXPath(root, xpath);
		String strPckLstID = XMLUtil.getValue(item.getAttributes().getNamedItem("PckLstID"));
		if(strPckLstID != "")
		{
			return Integer.parseInt(strPckLstID);
		}
		return 0;
	}

	private String getComponentName(int nFlPlnCmpntDefID) throws RemoteException, RecordManagerException, Exception
	{
		String xml = m_session.getDefinitionController().getComponentDefinitionById(nFlPlnCmpntDefID);
		Document doc = XMLUtil.getDocument(xml);
		Element root = doc.getDocumentElement();
		Node item = root.getFirstChild();
		return XMLUtil.getValue(item.getAttributes().getNamedItem("FlPlnCmpntDefNm"));
	}
	
	protected void checkDisplayStatus() throws Exception
	{
		if(m_DisplayItems == null)
		{
			throw new Exception("Assertion: Cannot use IRM Query Display methods if the bAddDisplayInfo constructor parameter is false!");
		}
	}
	
	protected void checkStatus() throws Exception
	{
		if(!m_bIsInitialized)
		{
			throw new Exception("Assertion: The Initialize method for the IRM Query object has not been called!");
		}
	}
}
